﻿using Rhino;
using Rhino.PlugIns;
using System.IO;
using System;
using Mit.Umi.RhinoServices.Context;
using Newtonsoft.Json;
using Mit.Umi.RhinoServices.UmiEvents;
using Mit.Umi.RhinoServices;
using System.Windows.Controls;
using System.Drawing.Imaging;
using System.Collections.Generic;
using System.Drawing;

namespace TransitCarbonDemo
{
    ///<summary>
    /// <para>Every RhinoCommon .rhp assembly must have one and only one PlugIn-derived
    /// class. DO NOT create instances of this class yourself. It is the
    /// responsibility of Rhino to create an instance of this class.</para>
    /// <para>To complete plug-in information, please also see all PlugInDescription
    /// attributes in AssemblyInfo.cs (you might need to click "Project" ->
    /// "Show All Files" to see it in the "Solution Explorer" window).</para>
    ///</summary>
    public class TransitCarbonDemoPlugIn : UmiModule
    {
        public TransitCarbonDemoPlugIn()
        {
            Instance = this;
        }

        ///<summary>Gets the only instance of the TransitCarbonDemoPlugIn plug-in.</summary>
        public static TransitCarbonDemoPlugIn Instance
        {
            get; private set;
        }

        // You can override methods here to change the plug-in behavior on
        // loading and shut down, add options pages to the Rhino _Option command
        // and mantain plug-in wide options in a document.

        public TransitCarbonSettings TransitCarbonSettings { get; set; } = new TransitCarbonSettings();

        public TransitCarbonPanel SimulatePanel { get; set; } = new TransitCarbonPanel();

        public Dictionary<Guid, double> Results { get; set; } = new Dictionary<Guid, double>();

        private void SaveSettings()
        {
            var context = UmiContext.Current;

            if (context == null)
            {
                return;
            }

            var json = JsonConvert.SerializeObject(TransitCarbonSettings);

            context.AuxiliaryFiles.StoreText("transit-carbon-settings.json", json);
        }

        private void LoadSettings(UmiContext context)
        {
            var path = context.AuxiliaryFiles.GetFullPath("transit-carbon-settings.json");

            var json = File.ReadAllText(path);

            TransitCarbonSettings = JsonConvert.DeserializeObject<TransitCarbonSettings>(json);
        }

        private void LoadResults(UmiContext context)
        {
            var objects = context.GetObjects();

            foreach (var obj in objects)
            {
                if (obj.Data.ContainsKey("Transit Carbon"))
                {
                    var id = Guid.Parse(obj.Id);
                    Results[id] = obj.Data["Transit Carbon"].Data[0];
                }
            }
        }

        protected override LoadReturnCode OnLoad(ref string errorMessage)
        {
            RhinoDoc.EndSaveDocument += RhinoDoc_EndSaveDocument;
            UmiEventSource.Instance.ProjectOpened += Instance_ProjectOpened;

            RhinoDoc.SelectObjects += RhinoDoc_SelectObjects;
            RhinoDoc.DeselectObjects += RhinoDoc_SelectObjects;

            return base.OnLoad(ref errorMessage);
        }

        protected override Color? Falsecolor(Guid objectId)
        {
            if (!Results.ContainsKey(objectId))
            {
                return null;
            }

            if (Results[objectId] > 100000)
            {
                return Color.Red;
            }
            else
            {
                return Color.Green;
            }
        }

        private void RhinoDoc_SelectObjects(object sender, Rhino.DocObjects.RhinoObjectSelectionEventArgs e)
        {
            var selected = RhinoDoc.ActiveDoc.Objects.GetSelectedObjects(includeGrips: false, includeLights: false);

            var total = 0.0;

            foreach (var obj in selected)
            {
                if (Results.ContainsKey(obj.Id))
                {
                    total += Results[obj.Id];
                }
            }

            SimulatePanel.textSelectedCarbon.Text = total.ToString();
        }

        private void Instance_ProjectOpened(object sender, UmiContext e)
        {
            LoadSettings(e);
            LoadResults(e);
        }

        private void RhinoDoc_EndSaveDocument(object sender, DocumentSaveEventArgs e)
        {
            SaveSettings();
        }

        public override PlugInLoadTime LoadTime
        {
            get
            {
                return PlugInLoadTime.AtStartup;
            }
        }

        protected override UserControl ModuleControl
        {
            get
            {
                return SimulatePanel;
            }
        }

        protected override string TabHeaderToolTip
        {
            get
            {
                return "Transit Carbon";
            }
        }

        protected override Tuple<System.Drawing.Bitmap, System.Drawing.Imaging.ImageFormat> TabHeaderIcon
        {
            get
            {
                return Tuple.Create(Properties.Resources.car_icon_hi, ImageFormat.Png);
            }
        }
    }
}